def GetThiTho(payload):
    """
    Function for displaying Thi and Tho paramameters, return as array
    payload - register from 141 to 156
    :return:
    """
    if len(payload) == 16:
        thitho=[]
        thiob=[f'{divmod(int(hex(payload[7]), 16), 256)[0]:08b}'[0:4], f'{divmod(int(hex(payload[7]), 16), 256)[0]:08b}'[4:]]
        thib=f'{divmod(int(hex(payload[7]), 16), 256)[1]:08b}'
        thob=f'{divmod(int(hex(payload[8]), 16), 256)[0]:08b}'
        thi = int(str(thiob[1])+str(thib), 2)
        if thi > 2047 :
            thi -= 4095
        thitho.append(thi/10)
        tho = int(str(thiob[0])+str(thob), 2)
        if tho > 2047 :
            tho -= 4095
        thitho.append(tho/10)
        return thitho
    else:
        return "Bad payload length"



def SetState(current,new):
    """
    function to change pump state

    :param current:
    :param new:
    :return:
    """
    try:
        newstate=[]
        if len(str(current[3])) <4 or len(str(current[0])) <4:
            return "Error"

        if new == "off":
            if int(hex(current[0])[4:6])%2 == 1:
                #state=int(hex(current[0])[4:6])-1
                state=int(100)
            else:
                return -1
        elif new == "on":
            if int(hex(current[0])[4:6])%2 == 0:
                #state=int(hex(current[0])[4:6])+1
                state=int(101)
            else:
                return -1
        elif new == "C":
            state=3
        elif new == "H":
            state=5
        elif new == "T":
            state=87
        elif new == "CT":
            state=83
        elif new == "HT":
            state=85
        else:
            return -1

        if state%2 == 0:
            num=256
        else:
            num=34304
        newstate.append(num+int(str(int(state)),16))
        newstate.append(current[1])
        newstate.append(int(hex(current[2])[2:6], 16))
        newstate.append(int(hex(current[3])[4:6], 16))
        newstate.append(int(hex(current[4])[4:6], 16))
        newstate.append(current[5])

        return newstate
    except:
        return "ERROR"



def GetDHWCurTemp(payload):
    """
    DHW current tank temperature display function
    :return:
    """
    if len(payload) == 16:
        dhwcurtemp = int(hex(payload[13]), 16)/10
        return dhwcurtemp
    else:
        return "Bad payload length"



def GetHeater(payload):
    """
    Function for displaying if heater is on or off
    payload - register from 141 to 157
    :return:
    """
    if len(payload) == 16:
        if payload[3] & 256:
            heater = 'ON'
        else:
            heater = 'OFF'
        return heater
    else:
        return "Bad payload length"



def GetFanRpm(payload):
    """
    Thanks to Daniel Mentel
    Function for displaying Fans rpms, return as array
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        fanrpm=[]
        fan1 = divmod(int(hex(payload[3]), 16), 256)[1]/2*10
        fan2 = divmod(int(hex(payload[4]), 16), 256)[0]/2*10
        fanrpm.append(fan1)
        fanrpm.append(fan2)
        return fanrpm
    else:
        return "Bad payload length"



def GetTao(payload):
    """
    Thanks to Daniel Mentel
    Function for displaying Tao parameter
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        tao=[]
        taomsb=f'{divmod(int(hex(payload[12]), 16), 256)[0]:08b}'[4:]
        taolsb=f'{divmod(int(hex(payload[12]), 16), 256)[1]:08b}'
        tao = int(str(taomsb)+str(taolsb), 2)
        if tao > 2047 :
            tao -= 4095
        return tao/10
    else:
        return "Bad payload length"



def GetArchError(payload):
    """
    Function for displaying archive errors 1,2,3, return as array
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        archerr=[]
        err1 = divmod(int(hex(payload[19]), 16), 256)[0]
        err2 = divmod(int(hex(payload[19]), 16), 256)[1]
        err3 = divmod(int(hex(payload[20]), 16), 256)[0]

        archerr.append(err1)
        archerr.append(err2)
        archerr.append(err3)
        return archerr
    else:
        return "Bad payload length"



def GetFirmware(payload):
    """
    Function for displaying current firmware version
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        firmware = divmod(int(hex(payload[18]), 16), 256)[0]/10
        return firmware
    else:
        return "Bad payload length"



def GetDHWTemp(payload):
    """
    DHW temperature display function
    :return:
    """
    if len(payload) == 6:
        dhwtemp = int(hex(payload[5])[2:4], 16)/2
        return dhwtemp
    else:
        return "Bad payload length"



def GetError(payload):
    """
    Function for displaying archive errors 1,2,3, return as array
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        error = divmod(int(hex(payload[1]), 16), 256)[1]

        return error
    else:
        return "Bad payload length"



def GetPump(payload):
    """
    Function for displaying if pump is on or off
    payload - register from 141 to 157
    :return:
    """
    if len(payload) == 16:
        tmp = divmod(int(hex(payload[3]), 16), 256)[0]
        if tmp != 16:
            pump = 'ON'
        else:
            pump = 'OFF'
        return pump
    else:
        return "Bad payload length"



def GetTSatPs(payload):
    """
    Thanks To Daniel Mentel
    Function for displaying saturation temperatures for target and actual Ps, returned as array
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        tab=[]
        tab_4bit=[f'{divmod(int(hex(payload[9]),16), 256)[0]:08b}'[4:], f'{divmod(int(hex(payload[9]), 16), 256)[0]:08b}'[0:4]]
        b53_8bit=f'{divmod(int(hex(payload[9]), 16), 256)[1]:08b}'
        b54_8bit=f'{divmod(int(hex(payload[10]), 16), 256)[0]:08b}'
        b53 = int(str(tab_4bit[0])+str(b53_8bit), 2)
        if (b53 > 2047) :
            b53 -= 4095
        tab.append(b53/10)
        b54 = int(str(tab_4bit[1])+str(b54_8bit), 2)
        if (b54 > 2047) :
            b54 -= 4095
        tab.append(b54/10)
        return tab
    else:
        return "Bad payload length"



def GetTdef(payload):
    """
    Function for displaying Tdef parameter
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        tdef=[]
        tdefmsb=f'{divmod(int(hex(payload[12]), 16), 256)[0]:08b}'[0:4]
        tdeflsb=f'{divmod(int(hex(payload[13]), 16), 256)[0]:08b}'
        tdef = int(str(tdefmsb)+str(tdeflsb), 2)
        if tdef > 2047 :
            tdef -= 4095
        return tdef/10
    else:
        return "Bad payload length"



def GetTwiTwo(payload):
    """
    Function for displaying Twi and Two paramameters, return as array
    payload - register from 141 to 156
    TODO: need to confirm its good, Two/Twi cannot be more than 51.1°C
    :return:
    """
    if len(payload) == 16:
        twitwo=[]
        twiob=[f'{divmod(int(hex(payload[5]), 16), 256)[1]:08b}'[0:4], f'{divmod(int(hex(payload[5]), 16), 256)[1]:08b}'[4:]]
        twib=f'{divmod(int(hex(payload[6]), 16), 256)[0]:08b}'
        twob=f'{divmod(int(hex(payload[6]), 16), 256)[1]:08b}'
        twi = int(str(twiob[1])+str(twib), 2)/10
        twitwo.append(twi)
        two = int(str(twiob[0])+str(twob), 2)/10
        twitwo.append(two)
        return twitwo
    else:
        return "Bad payload length"
